import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread";
import { moveRegion } from "../../ImageCanvas/region-tools.js";
import { getIn, setIn } from "seamless-immutable";
import isEqual from "lodash/isEqual";
import getActiveImage from "./get-active-image";
import { saveToHistory } from "./history-handler.js";
import colors from "../../colors";
import fixTwisted from "./fix-twisted";
import convertExpandingLineToPolygon from "./convert-expanding-line-to-polygon";
import clamp from "clamp";
import getLandmarksWithTransform from "../../utils/get-landmarks-with-transform";
import setInLocalStorage from "../../utils/set-in-local-storage";
import calculateDistance from "../../utils/calculateDistance";
import { getFunctions, httpsCallable } from "firebase/functions";
import { initializeApp } from "firebase/app";

var getRandomId = function getRandomId() {
  return Math.random().toString().split(".")[1];
};

export default (function (state, action) {
  if (state.allowedArea && state.selectedTool !== "modify-allowed-area" && ["MOUSE_DOWN", "MOUSE_UP", "MOUSE_MOVE"].includes(action.type)) {
    var aa = state.allowedArea;
    action.x = clamp(action.x, aa.x, aa.x + aa.w);
    action.y = clamp(action.y, aa.y, aa.y + aa.h);
  }

  if (action.type === "ON_CLS_ADDED" && action.cls && action.cls !== "") {
    var oldRegionClsList = state.regionClsList;

    var newState = _objectSpread({}, state, {
      regionClsList: oldRegionClsList.concat(action.cls)
    });

    return newState;
  } // Throttle certain actions


  if (action.type === "MOUSE_MOVE") {
    if (Date.now() - (state.lastMouseMoveCall || 0) < 16) return state;
    state = setIn(state, ["lastMouseMoveCall"], Date.now());
  }

  if (!action.type.includes("MOUSE")) {
    state = setIn(state, ["lastAction"], action);
  }

  var _getActiveImage = getActiveImage(state),
      currentImageIndex = _getActiveImage.currentImageIndex,
      pathToActiveImage = _getActiveImage.pathToActiveImage,
      activeImage = _getActiveImage.activeImage;

  var getRegionIndex = function getRegionIndex(region) {
    var regionId = typeof region === "string" || typeof region === "number" ? region : region.id;
    if (!activeImage) return null;
    var regionIndex = (activeImage.regions || []).findIndex(function (r) {
      return r.id === regionId;
    });
    return regionIndex === -1 ? null : regionIndex;
  };

  var getRegion = function getRegion(regionId) {
    if (!activeImage) return null;
    var regionIndex = getRegionIndex(regionId);
    if (regionIndex === null) return [null, null];
    var region = activeImage.regions[regionIndex];
    return [region, regionIndex];
  };

  var modifyRegion = function modifyRegion(regionId, obj) {
    var _getRegion = getRegion(regionId),
        _getRegion2 = _slicedToArray(_getRegion, 2),
        region = _getRegion2[0],
        regionIndex = _getRegion2[1];

    if (!region) return state;

    if (obj !== null) {
      return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", regionIndex]), _objectSpread({}, region, obj));
    } else {
      // delete region
      var regions = activeImage.regions;
      return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions"]), (regions || []).filter(function (r) {
        return r.id !== region.id;
      }));
    }
  }; // eslint-disable-next-line


  var unselectRegions = function unselectRegions(state) {
    if (!activeImage) return state;
    return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions"]), (activeImage.regions || []).map(function (r) {
      return _objectSpread({}, r, {
        highlighted: false
      });
    }));
  };

  var closeEditors = function closeEditors(state) {
    if (currentImageIndex === null) return state;
    return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions"]), (activeImage.regions || []).map(function (r) {
      return _objectSpread({}, r, {
        editingLabels: false
      });
    }));
  };

  var setNewImage = function setNewImage(img, index) {
    // eslint-disable-next-line
    var _ref = typeof img === "object" ? img : {
      src: img
    },
        src = _ref.src,
        frameTime = _ref.frameTime;

    return setIn(setIn(state, ["selectedImage"], index), ["selectedImageFrameTime"], frameTime);
  };

  switch (action.type) {
    case "@@INIT":
      {
        return state;
      }

    case "SELECT_IMAGE":
      {
        return setNewImage(action.image, action.imageIndex);
      }

    case "SELECT_CLASSIFICATION":
      {
        return setIn(state, ["selectedCls"], action.cls);
      }

    case "CREATE_NEW_POLYLINE_BS":
      {
        var region = action.region,
            pointIndex = action.pointIndex;
        var newRegion;
        var defaultRegionCls = state.selectedCls,
            defaultRegionColor = "#ff0000";
        var point = region.points[pointIndex];
        state = saveToHistory(state, "Create Polyline BS");
        newRegion = {
          type: "polyline-bs",
          points: [point, point],
          pointsWithCls: [defaultRegionCls, defaultRegionCls],
          highlighted: true,
          color: defaultRegionColor,
          cls: defaultRegionCls,
          id: getRandomId()
        };
        state = setIn(state, ["mode"], {
          mode: "DRAW_POLYLINE_BS",
          regionId: newRegion.id
        });

        var regions = _toConsumableArray(getIn(state, pathToActiveImage).regions || []).map(function (r) {
          return setIn(r, ["editingLabels"], false).setIn(["highlighted"], false);
        }).concat(newRegion ? [newRegion] : []);

        return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions"]), regions);
      }

    case "PROPAGATE":
      {
        var copyCount = action.copyCount;
        var selectedRegion = state.images[state.selectedImage].regions.find(function (r) {
          return r.highlighted;
        });

        var _newRegion;

        if (selectedRegion) {
          _newRegion = _objectSpread({}, selectedRegion, {
            highlighted: true,
            editingLabels: false,
            id: getRandomId()
          });
        }

        var slicedImages = state.images.slice(state.selectedImage + 1, state.images.length - 1).slice(0, copyCount);
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          var _loop = function _loop() {
            var img = _step.value;
            var imgIndex = state.images.findIndex(function (i) {
              return i.name === img.name;
            });

            var regions = _toConsumableArray(getIn(state, ["images", imgIndex]).regions || []).map(function (r) {
              return setIn(r, ["editingLabels"], false).setIn(["highlighted"], false);
            }).concat(_newRegion ? [_newRegion] : []);

            state = setIn(state, ["images", imgIndex, "regions"], regions);
          };

          for (var _iterator = slicedImages[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            _loop();
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        return state;
      }

    case "CHANGE_REGION":
      {
        var regionIndex = getRegionIndex(action.region);
        if (regionIndex === null) return state;
        var oldRegion = activeImage.regions[regionIndex];

        if (oldRegion.cls !== action.region.cls) {
          state = saveToHistory(state, "Change Region Classification");
          var clsIndex = state.regionClsList.indexOf(action.region.cls);

          if (clsIndex !== -1) {
            state = setIn(state, ["selectedCls"], action.region.cls);
            action.region.color = colors[clsIndex % colors.length];
          }
        }

        if (!isEqual(oldRegion.tags, action.region.tags)) {
          state = saveToHistory(state, "Change Region Tags");
        }

        if (!isEqual(oldRegion.comment, action.region.comment)) {
          state = saveToHistory(state, "Change Region Comment");
        }

        return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", regionIndex]), action.region);
      }

    case "CHANGE_REGION_POINTS":
      {
        var _region = action.region,
            subClassMenuData = action.subClassMenuData;

        var _regionIndex = getRegionIndex(_region); // console.log(subClassMenuData)


        var newPointsWithCls = _region.pointsWithCls.map(function (point, i) {
          if (i === subClassMenuData.pointIndex) return _region.cls;
          return point;
        });

        return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex, "pointsWithCls"]), newPointsWithCls); // return state
      }

    case "CHANGE_IMAGE":
      {
        if (!activeImage) return state;
        var delta = action.delta;

        for (var _i = 0, _Object$keys = Object.keys(delta); _i < _Object$keys.length; _i++) {
          var key = _Object$keys[_i];
          if (key === "cls") saveToHistory(state, "Change Image Class");
          if (key === "tags") saveToHistory(state, "Change Image Tags");
          state = setIn(state, [].concat(_toConsumableArray(pathToActiveImage), [key]), delta[key]);
        }

        return state;
      }

    case "SELECT_REGION":
      {
        var _region2 = action.region;

        var _regionIndex2 = getRegionIndex(action.region);

        if (_regionIndex2 === null) return state;

        var _regions = _toConsumableArray(activeImage.regions || []).map(function (r) {
          return _objectSpread({}, r, {
            highlighted: r.id === _region2.id,
            editingLabels: r.id === _region2.id
          });
        });

        return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions"]), _regions);
      }

    case "BEGIN_MOVE_POINT":
      {
        state = closeEditors(state);
        return setIn(state, ["mode"], {
          mode: "MOVE_REGION",
          regionId: action.point.id
        });
      }

    case "BEGIN_BOX_TRANSFORM":
      {
        var box = action.box,
            directions = action.directions;
        state = closeEditors(state);

        if (directions[0] === 0 && directions[1] === 0) {
          return setIn(state, ["mode"], {
            mode: "MOVE_REGION",
            regionId: box.id
          });
        } else {
          return setIn(state, ["mode"], {
            mode: "RESIZE_BOX",
            regionId: box.id,
            freedom: directions,
            original: {
              x: box.x,
              y: box.y,
              w: box.w,
              h: box.h
            }
          });
        }
      }

    case "BEGIN_MOVE_POLYGON_POINT":
      {
        var polygon = action.polygon,
            _pointIndex = action.pointIndex;
        state = closeEditors(state);

        if (state.mode && state.mode.mode === "DRAW_POLYGON" && _pointIndex === 0) {
          return setIn(modifyRegion(polygon, {
            points: polygon.points.slice(0, -1),
            open: false,
            editingLabels: state.selectedCls ? false : true
          }), ["mode"], null);
        } else {
          state = saveToHistory(state, "Move Polygon Point");
        }

        return setIn(state, ["mode"], {
          mode: "MOVE_POLYGON_POINT",
          regionId: polygon.id,
          pointIndex: _pointIndex
        });
      }

    case "FINISH_POLYGON":
      {
        state = closeEditors(state);

        if (state.mode && state.mode.mode === "DRAW_POLYGON") {
          var _polygon = state.images[state.selectedImage].regions.find(function (r) {
            return r.id === state.mode.regionId;
          }); // if (polygon.points.length > 3)


          return setIn(modifyRegion(_polygon, {
            points: _polygon.points.slice(0, -1),
            open: false,
            editingLabels: state.selectedCls ? false : true
          }), ["mode"], null);
        }
      }
    // eslint-disable-next-line

    case "FINISH_POLYLINE":
      {
        var selectedRegions = state.images[state.selectedImage].regions;

        if (state.mode && state.mode.mode === "DRAW_POLYLINE" && state.selectedTool === "create-polyline") {
          if (selectedRegions) {
            var polyline = selectedRegions.find(function (r) {
              return r.highlighted;
            });

            if (polyline) {
              return setIn(modifyRegion(polyline, {
                points: polyline.points.slice(0, -1),
                editingLabels: state.selectedCls ? false : true,
                finished: true
              }), ["mode"], null);
            }
          }
        }

        break;
      }

    case "FINISH_POLYLINE_BS":
      {
        var _selectedRegions = state.images[state.selectedImage].regions;

        if (state.mode && state.selectedTool === "create-polyline-bs") {
          if (_selectedRegions) {
            var _polyline = _selectedRegions.find(function (r) {
              return r.highlighted;
            });

            if (_polyline) {
              return setIn(modifyRegion(_polyline, {
                points: _polyline.points.slice(0, -1),
                editingLabels: state.selectedCls ? false : true,
                finished: true
              }), ["mode"], null);
            }
          }
        }

        break;
      }

    case "FINISH_AI":
      {
        var _selectedRegions2 = state.images[state.selectedImage].regions;

        if (state.selectedTool === "create-ai") {
          console.log(_selectedRegions2);
        }

        break;
      }

    case "HANDLE_PREVENT":
      {
        var event = action.event;

        if (state.mode && state.mode.mode.includes("DRAW")) {
          event.preventDefault();
        }

        break;
      }

    case "BEGIN_MOVE_KEYPOINT":
      {
        var _region3 = action.region,
            keypointId = action.keypointId;
        state = closeEditors(state);
        state = saveToHistory(state, "Move Keypoint");
        return setIn(state, ["mode"], {
          mode: "MOVE_KEYPOINT",
          regionId: _region3.id,
          keypointId: keypointId
        });
      }

    case "ADD_POLYGON_POINT":
      {
        // console.log("as")
        var _polygon2 = action.polygon,
            _point = action.point,
            _pointIndex2 = action.pointIndex;

        var _regionIndex3 = getRegionIndex(_polygon2);

        if (_regionIndex3 === null) return state;

        var points = _toConsumableArray(_polygon2.points);

        points.splice(_pointIndex2, 0, _point);
        return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex3]), _objectSpread({}, _polygon2, {
          points: points
        }));
      }

    case "MOUSE_MOVE":
      {
        var x = action.x,
            y = action.y,
            mat = action.mat,
            isShiftDown = action.isShiftDown;
        if (!state.mode) return state;
        if (!activeImage) return state; // const { mouseDownAt } = state

        switch (state.mode.mode) {
          case "MOVE_POLYGON_POINT":
            {
              var _state$mode = state.mode,
                  _pointIndex3 = _state$mode.pointIndex,
                  regionId = _state$mode.regionId;

              var _regionIndex4 = getRegionIndex(regionId);

              if (_regionIndex4 === null) return state;
              return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex4, "points", _pointIndex3]), [x, y]);
            }

          case "MOVE_KEYPOINT":
            {
              var _state$mode2 = state.mode,
                  _keypointId = _state$mode2.keypointId,
                  _regionId = _state$mode2.regionId;

              var _getRegion3 = getRegion(_regionId),
                  _getRegion4 = _slicedToArray(_getRegion3, 2),
                  _region4 = _getRegion4[0],
                  _regionIndex5 = _getRegion4[1];

              if (_regionIndex5 === null) return state;
              return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex5, "points", _keypointId]), _objectSpread({}, _region4.points[_keypointId], {
                x: x,
                y: y
              }));
            }

          case "MOVE_REGION":
            {
              var _regionId2 = state.mode.regionId;

              if (_regionId2 === "$$allowed_area") {
                var _state = state,
                    _state$allowedArea = _state.allowedArea,
                    w = _state$allowedArea.w,
                    h = _state$allowedArea.h;
                return setIn(state, ["allowedArea"], {
                  x: x - w / 2,
                  y: y - h / 2,
                  w: w,
                  h: h
                });
              }

              var _regionIndex6 = getRegionIndex(_regionId2);

              if (_regionIndex6 === null) return state;
              return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex6]), moveRegion(activeImage.regions[_regionIndex6], x, y));
            }

          case "RESIZE_BOX":
            {
              var _state$mode3 = state.mode,
                  _regionId3 = _state$mode3.regionId,
                  _state$mode3$freedom = _slicedToArray(_state$mode3.freedom, 2),
                  xFree = _state$mode3$freedom[0],
                  yFree = _state$mode3$freedom[1],
                  _state$mode3$original = _state$mode3.original,
                  ox = _state$mode3$original.x,
                  oy = _state$mode3$original.y,
                  ow = _state$mode3$original.w,
                  oh = _state$mode3$original.h;

              var dx = xFree === 0 ? ox : xFree === -1 ? Math.min(ox + ow, x) : ox;
              var dw = xFree === 0 ? ow : xFree === -1 ? ow + (ox - dx) : Math.max(0, ow + (x - ox - ow));
              var dy = yFree === 0 ? oy : yFree === -1 ? Math.min(oy + oh, y) : oy;
              var dh = yFree === 0 ? oh : yFree === -1 ? oh + (oy - dy) : Math.max(0, oh + (y - oy - oh)); // determine if we should switch the freedom

              if (dw <= 0.001) {
                state = setIn(state, ["mode", "freedom"], [xFree * -1, yFree]);
              }

              if (dh <= 0.001) {
                state = setIn(state, ["mode", "freedom"], [xFree, yFree * -1]);
              }

              if (_regionId3 === "$$allowed_area") {
                return setIn(state, ["allowedArea"], {
                  x: dx,
                  w: dw,
                  y: dy,
                  h: dh
                });
              }

              var _regionIndex7 = getRegionIndex(_regionId3);

              if (_regionIndex7 === null) return state;
              var _box = activeImage.regions[_regionIndex7];
              return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex7]), _objectSpread({}, _box, {
                x: dx,
                w: dw,
                y: dy,
                h: dh
              }));
            }

          case "RESIZE_KEYPOINTS":
            {
              var _state$mode4 = state.mode,
                  _regionId4 = _state$mode4.regionId,
                  landmarks = _state$mode4.landmarks,
                  centerX = _state$mode4.centerX,
                  centerY = _state$mode4.centerY;
              var distFromCenter = Math.sqrt(Math.pow(centerX - x, 2) + Math.pow(centerY - y, 2));
              var scale = distFromCenter / 0.15;
              return modifyRegion(_regionId4, {
                points: getLandmarksWithTransform({
                  landmarks: landmarks,
                  center: {
                    x: centerX,
                    y: centerY
                  },
                  scale: scale
                })
              });
            }

          case "DRAW_POLYGON":
            {
              // console.log("polygon moving")
              var _regionId5 = state.mode.regionId;

              var _getRegion5 = getRegion(_regionId5),
                  _getRegion6 = _slicedToArray(_getRegion5, 2),
                  _region5 = _getRegion6[0],
                  _regionIndex8 = _getRegion6[1];

              if (!_region5) return setIn(state, ["mode"], null);

              if (isShiftDown) {
                var selectedImage = state.images[state.selectedImage];
                var _ref2 = [selectedImage.pixelSize.w, selectedImage.pixelSize.h],
                    imgWidth = _ref2[0],
                    imgHeight = _ref2[1];

                var _region5$points = _slicedToArray(_region5.points[_region5.points.length - 2], 2),
                    lastX = _region5$points[0],
                    lastY = _region5$points[1];

                var distanceBetweenTwoPoints = calculateDistance(x * imgWidth, y * imgHeight, lastX * imgWidth, lastY * imgHeight);
                var zoomScale = 1 / mat.a;

                if (Number(20 / zoomScale) <= Number(distanceBetweenTwoPoints)) {
                  state = saveToHistory(state, "Create Point");
                  return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex8]), _objectSpread({}, _region5, {
                    points: _region5.points.concat([[x, y]])
                  }));
                }
              }

              return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex8, "points", _region5.points.length - 1]), [x, y]);
            }

          case "DRAW_POLYLINE":
            {
              // console.log("polyline moving", x,y)
              var _regionId6 = state.mode.regionId;

              var _getRegion7 = getRegion(_regionId6),
                  _getRegion8 = _slicedToArray(_getRegion7, 2),
                  _region6 = _getRegion8[0],
                  _regionIndex9 = _getRegion8[1];

              if (!_region6) return setIn(state, ["mode"], null);
              return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex9, "points", _region6.points.length - 1]), [x, y]);
            }

          case "DRAW_POLYLINE_BS":
            {
              // console.log("polyline moving", x,y)
              var _regionId7 = state.mode.regionId;

              var _getRegion9 = getRegion(_regionId7),
                  _getRegion10 = _slicedToArray(_getRegion9, 2),
                  _region7 = _getRegion10[0],
                  _regionIndex10 = _getRegion10[1];

              if (!_region7) return setIn(state, ["mode"], null);
              return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex10, "points", _region7.points.length - 1]), [x, y]);
            }

          case "DRAW_LINE":
            {
              var _regionId8 = state.mode.regionId;

              var _getRegion11 = getRegion(_regionId8),
                  _getRegion12 = _slicedToArray(_getRegion11, 2),
                  _region8 = _getRegion12[0],
                  _regionIndex11 = _getRegion12[1];

              if (!_region8) return setIn(state, ["mode"], null);
              return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex11]), _objectSpread({}, _region8, {
                x2: x,
                y2: y
              }));
            }

          case "DRAW_EXPANDING_LINE":
            {
              var _regionId9 = state.mode.regionId;

              var _getRegion13 = getRegion(_regionId9),
                  _getRegion14 = _slicedToArray(_getRegion13, 2),
                  expandingLine = _getRegion14[0],
                  _regionIndex12 = _getRegion14[1];

              if (!expandingLine) return state;
              var isMouseDown = Boolean(state.mouseDownAt);

              if (isMouseDown) {
                // If the mouse is down, set width/angle
                var lastPoint = expandingLine.points.slice(-1)[0];
                var mouseDistFromLastPoint = Math.sqrt(Math.pow(lastPoint.x - x, 2) + Math.pow(lastPoint.y - y, 2));
                if (mouseDistFromLastPoint < 0.002 && !lastPoint.width) return state;

                var _newState = setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex12, "points"]), expandingLine.points.slice(0, -1).concat([_objectSpread({}, lastPoint, {
                  width: mouseDistFromLastPoint * 2,
                  angle: Math.atan2(lastPoint.x - x, lastPoint.y - y)
                })]));

                return _newState;
              } else {
                // If mouse is up, move the next candidate point
                return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex12]), _objectSpread({}, expandingLine, {
                  candidatePoint: {
                    x: x,
                    y: y
                  }
                }));
              } // eslint-disable-next-line


              return state;
            }

          case "SET_EXPANDING_LINE_WIDTH":
            {
              var _regionId10 = state.mode.regionId;

              var _getRegion15 = getRegion(_regionId10),
                  _getRegion16 = _slicedToArray(_getRegion15, 2),
                  _expandingLine = _getRegion16[0],
                  _regionIndex13 = _getRegion16[1];

              if (!_expandingLine) return state;

              var _lastPoint = _expandingLine.points.slice(-1)[0]; // const { mouseDownAt } = state


              return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex13, "expandingWidth"]), Math.sqrt(Math.pow(_lastPoint.x - x, 2) + Math.pow(_lastPoint.y - y, 2)));
            }

          default:
            return state;
        }
      }

    case "MOUSE_DOWN":
      {
        if (!activeImage) return state;
        var _x = action.x,
            _y = action.y;
        state = setIn(state, ["mouseDownAt"], {
          x: _x,
          y: _y
        });

        if (state.mode) {
          switch (state.mode.mode) {
            case "DRAW_POLYGON":
              {
                // console.log("draw polygon line")
                var _getRegion17 = getRegion(state.mode.regionId),
                    _getRegion18 = _slicedToArray(_getRegion17, 2),
                    _polygon3 = _getRegion18[0],
                    _regionIndex14 = _getRegion18[1];

                if (!_polygon3) break;
                state = saveToHistory(state, "Create Point");
                return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex14]), _objectSpread({}, _polygon3, {
                  points: _polygon3.points.concat([[_x, _y]])
                }));
              }

            case "DRAW_POLYLINE":
              {
                // console.log("draw polyline line")
                var _getRegion19 = getRegion(state.mode.regionId),
                    _getRegion20 = _slicedToArray(_getRegion19, 2),
                    _polyline2 = _getRegion20[0],
                    _regionIndex15 = _getRegion20[1];

                if (!_polyline2) break;
                return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex15]), _objectSpread({}, _polyline2, {
                  points: _polyline2.points.concat([[_x, _y]])
                }));
              }

            case "DRAW_POLYLINE_BS":
              {
                // console.log("draw polyline line bs")
                var _getRegion21 = getRegion(state.mode.regionId),
                    _getRegion22 = _slicedToArray(_getRegion21, 2),
                    _polyline3 = _getRegion22[0],
                    _regionIndex16 = _getRegion22[1];

                if (!_polyline3) break;
                return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex16]), _objectSpread({}, _polyline3, {
                  points: _polyline3.points.concat([[_x, _y]]),
                  pointsWithCls: _polyline3.pointsWithCls.concat([""])
                }));
              }

            case "DRAW_LINE":
              {
                var _getRegion23 = getRegion(state.mode.regionId),
                    _getRegion24 = _slicedToArray(_getRegion23, 2),
                    line = _getRegion24[0],
                    _regionIndex17 = _getRegion24[1];

                if (!line) break;
                setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex17]), _objectSpread({}, line, {
                  x2: _x,
                  y2: _y
                }));
                return setIn(state, ["mode"], null);
              }

            case "DRAW_EXPANDING_LINE":
              {
                var _getRegion25 = getRegion(state.mode.regionId),
                    _getRegion26 = _slicedToArray(_getRegion25, 2),
                    _expandingLine2 = _getRegion26[0],
                    _regionIndex18 = _getRegion26[1];

                if (!_expandingLine2) break;

                var _lastPoint2 = _expandingLine2.points.slice(-1)[0];

                if (_expandingLine2.points.length > 1 && Math.sqrt(Math.pow(_lastPoint2.x - _x, 2) + Math.pow(_lastPoint2.y - _y, 2)) < 0.002) {
                  if (!_lastPoint2.width) {
                    return setIn(state, ["mode"], {
                      mode: "SET_EXPANDING_LINE_WIDTH",
                      regionId: state.mode.regionId
                    });
                  } else {
                    return state.setIn([].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex18]), convertExpandingLineToPolygon(_expandingLine2)).setIn(["mode"], null);
                  }
                } // Create new point


                return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex18, "points"]), _expandingLine2.points.concat([{
                  x: _x,
                  y: _y,
                  angle: null,
                  width: null
                }]));
              }

            case "SET_EXPANDING_LINE_WIDTH":
              {
                var _getRegion27 = getRegion(state.mode.regionId),
                    _getRegion28 = _slicedToArray(_getRegion27, 2),
                    _expandingLine3 = _getRegion28[0],
                    _regionIndex19 = _getRegion28[1];

                if (!_expandingLine3) break;
                var expandingWidth = _expandingLine3.expandingWidth;
                return state.setIn([].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex19]), convertExpandingLineToPolygon(_objectSpread({}, _expandingLine3, {
                  points: _expandingLine3.points.map(function (p) {
                    return p.width ? p : _objectSpread({}, p, {
                      width: expandingWidth
                    });
                  }),
                  expandingWidth: undefined
                }))).setIn(["mode"], null);
              }

            default:
              break;
          }
        }

        var _newRegion2;

        var _defaultRegionCls = state.selectedCls,
            _defaultRegionColor = "#ff0000";

        var _clsIndex = (state.regionClsList || []).indexOf(_defaultRegionCls);

        if (_clsIndex !== -1) {
          _defaultRegionColor = colors[_clsIndex % colors.length];
        }

        switch (state.selectedTool) {
          case "create-ai":
            {
              state = saveToHistory(state, "Create AI Point");
              _newRegion2 = {
                type: "ai-point",
                x: _x,
                y: _y,
                highlighted: false,
                editingLabels: false,
                color: _defaultRegionColor,
                id: getRandomId(),
                cls: undefined
              };
              var _selectedImage = state.images[state.selectedImage];
              var firebaseApp = state.firebaseConfig ? initializeApp(state.firebaseConfig) : null;
              var functions = getFunctions(firebaseApp);
              var promptSAM = httpsCallable(functions, "promptSAM");
              promptSAM({
                imageURL: _selectedImage.src,
                prompts: [{
                  inclusivePoints: [{
                    x: _x * _selectedImage.pixelSize.w,
                    y: _y * _selectedImage.pixelSize.h
                  }],
                  exclusivePoints: []
                }]
              }).then(function (_ref3) {
                var data = _ref3.data;
                console.log(data);
              });
              break;
            }

          case "create-point":
            {
              state = saveToHistory(state, "Create Point");
              _newRegion2 = {
                type: "point",
                x: _x,
                y: _y,
                highlighted: true,
                editingLabels: true,
                color: _defaultRegionColor,
                id: getRandomId(),
                cls: _defaultRegionCls
              };
              break;
            }

          case "create-box":
            {
              state = saveToHistory(state, "Create Box");
              _newRegion2 = {
                type: "box",
                x: _x,
                y: _y,
                w: 0,
                h: 0,
                highlighted: true,
                editingLabels: false,
                color: _defaultRegionColor,
                cls: _defaultRegionCls,
                id: getRandomId()
              };
              state = setIn(state, ["mode"], {
                mode: "RESIZE_BOX",
                editLabelEditorAfter: true,
                regionId: _newRegion2.id,
                freedom: [1, 1],
                original: {
                  x: _x,
                  y: _y,
                  w: _newRegion2.w,
                  h: _newRegion2.h
                },
                isNew: true
              });
              break;
            }

          case "create-polygon":
            {
              // console.log("create first polygon point")
              if (state.mode && state.mode.mode === "DRAW_POLYGON") break;
              state = saveToHistory(state, "Create Polygon");
              _newRegion2 = {
                type: "polygon",
                points: [[_x, _y], [_x, _y]],
                open: true,
                highlighted: true,
                color: _defaultRegionColor,
                cls: _defaultRegionCls,
                id: getRandomId()
              };
              state = setIn(state, ["mode"], {
                mode: "DRAW_POLYGON",
                regionId: _newRegion2.id
              });
              break;
            }

          case "create-polyline-bs":
            {
              state = saveToHistory(state, "Create Polyline BS");
              _newRegion2 = {
                type: "polyline-bs",
                points: [[_x, _y], [_x, _y]],
                pointsWithCls: ["", ""],
                highlighted: true,
                color: _defaultRegionColor,
                cls: _defaultRegionCls,
                id: getRandomId()
              };
              state = setIn(state, ["mode"], {
                mode: "DRAW_POLYLINE_BS",
                regionId: _newRegion2.id
              });
              break;
            }

          case "create-polyline":
            {
              if (state.mode && state.mode.mode === "DRAW_POLYLINE") break;
              state = saveToHistory(state, "Create Polyline");
              _newRegion2 = {
                type: "polyline",
                points: [[_x, _y], [_x, _y]],
                highlighted: true,
                color: _defaultRegionColor,
                cls: _defaultRegionCls,
                id: getRandomId()
              };
              state = setIn(state, ["mode"], {
                mode: "DRAW_POLYLINE",
                regionId: _newRegion2.id
              });
              break;
            }

          case "create-expanding-line":
            {
              state = saveToHistory(state, "Create Expanding Line");
              _newRegion2 = {
                type: "expanding-line",
                unfinished: true,
                points: [{
                  x: _x,
                  y: _y,
                  angle: null,
                  width: null
                }],
                open: true,
                highlighted: true,
                color: _defaultRegionColor,
                cls: _defaultRegionCls,
                id: getRandomId()
              };
              state = setIn(state, ["mode"], {
                mode: "DRAW_EXPANDING_LINE",
                regionId: _newRegion2.id
              });
              break;
            }

          case "create-line":
            {
              if (state.mode && state.mode.mode === "DRAW_LINE") break;
              state = saveToHistory(state, "Create Line");
              _newRegion2 = {
                type: "line",
                x1: _x,
                y1: _y,
                x2: _x,
                y2: _y,
                highlighted: true,
                editingLabels: false,
                color: _defaultRegionColor,
                cls: _defaultRegionCls,
                id: getRandomId()
              };
              state = setIn(state, ["mode"], {
                mode: "DRAW_LINE",
                regionId: _newRegion2.id
              });
              break;
            }

          case "create-keypoints":
            {
              state = saveToHistory(state, "Create Keypoints"); // eslint-disable-next-line

              var _ref4 = Object.entries(state.keypointDefinitions),
                  _ref5 = _slicedToArray(_ref4, 1),
                  _ref5$ = _slicedToArray(_ref5[0], 2),
                  keypointsDefinitionId = _ref5$[0],
                  _ref5$$ = _ref5$[1],
                  _landmarks = _ref5$$.landmarks,
                  connections = _ref5$$.connections;

              _newRegion2 = {
                type: "keypoints",
                keypointsDefinitionId: keypointsDefinitionId,
                points: getLandmarksWithTransform({
                  landmarks: _landmarks,
                  center: {
                    x: _x,
                    y: _y
                  },
                  scale: 1
                }),
                highlighted: true,
                editingLabels: false,
                id: getRandomId()
              };
              state = setIn(state, ["mode"], {
                mode: "RESIZE_KEYPOINTS",
                landmarks: _landmarks,
                centerX: _x,
                centerY: _y,
                regionId: _newRegion2.id,
                isNew: true
              });
              break;
            }

          default:
            break;
        }

        if (_newRegion2) _newRegion2.showComment = false;

        var _regions2 = _toConsumableArray(getIn(state, pathToActiveImage).regions || []).map(function (r) {
          return setIn(r, ["editingLabels"], false).setIn(["highlighted"], false);
        }).concat(_newRegion2 ? [_newRegion2] : []);

        return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions"]), _regions2);
      }

    case "MOUSE_RIGHT_DOWN":
      {
        if (!activeImage) return state;
        if (state.selectedTool !== "create-ai") return state;
        var _x2 = action.x,
            _y2 = action.y;

        var _newRegion3;

        state = saveToHistory(state, "Create AI RM Point");
        _newRegion3 = {
          type: "ai-rm-point",
          x: _x2,
          y: _y2,
          highlighted: false,
          editingLabels: false,
          color: "red",
          id: getRandomId(),
          cls: undefined
        };
        if (_newRegion3) _newRegion3.showComment = false;

        var _regions3 = _toConsumableArray(getIn(state, pathToActiveImage).regions || []).map(function (r) {
          return setIn(r, ["editingLabels"], false).setIn(["highlighted"], false);
        }).concat(_newRegion3 ? [_newRegion3] : []);

        return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions"]), _regions3);
      }

    case "MOUSE_UP":
      {
        var _x3 = action.x,
            _y3 = action.y;
        var _state2 = state,
            _state2$mouseDownAt = _state2.mouseDownAt,
            mouseDownAt = _state2$mouseDownAt === void 0 ? {
          x: _x3,
          y: _y3
        } : _state2$mouseDownAt;
        if (!state.mode) return state;
        state = setIn(state, ["mouseDownAt"], null);

        switch (state.mode.mode) {
          case "RESIZE_BOX":
            {
              if (state.mode.isNew) {
                if (Math.abs(state.mode.original.x - _x3) < 0.002 || Math.abs(state.mode.original.y - _y3) < 0.002) {
                  return setIn(modifyRegion(state.mode.regionId, null), ["mode"], null);
                }
              }

              if (state.mode.editLabelEditorAfter) {
                return _objectSpread({}, modifyRegion(state.mode.regionId, {
                  editingLabels: true
                }), {
                  mode: null
                });
              }
            }
          // eslint-disable-next-line

          case "MOVE_REGION":
          case "RESIZE_KEYPOINTS":
          case "MOVE_POLYGON_POINT":
            {
              // console.log("sa")
              return _objectSpread({}, state, {
                mode: null
              });
            }

          case "MOVE_KEYPOINT":
            {
              return _objectSpread({}, state, {
                mode: null
              });
            }

          case "CREATE_POINT_LINE":
            {
              return state;
            }

          case "DRAW_EXPANDING_LINE":
            {
              var _getRegion29 = getRegion(state.mode.regionId),
                  _getRegion30 = _slicedToArray(_getRegion29, 2),
                  _expandingLine4 = _getRegion30[0],
                  _regionIndex20 = _getRegion30[1];

              if (!_expandingLine4) return state;
              var newExpandingLine = _expandingLine4;

              var _lastPoint3 = _expandingLine4.points.length !== 0 ? _expandingLine4.points.slice(-1)[0] : mouseDownAt; // eslint-disable-next-line


              var jointStart;

              if (_expandingLine4.points.length > 1) {
                jointStart = _expandingLine4.points.slice(-2)[0];
              } else {
                jointStart = _lastPoint3;
              }

              var _mouseDistFromLastPoint = Math.sqrt(Math.pow(_lastPoint3.x - _x3, 2) + Math.pow(_lastPoint3.y - _y3, 2));

              if (_mouseDistFromLastPoint > 0.002) {
                // The user is drawing has drawn the width for the last point
                var newPoints = _toConsumableArray(_expandingLine4.points);

                for (var i = 0; i < newPoints.length - 1; i++) {
                  if (newPoints[i].width) continue;
                  newPoints[i] = _objectSpread({}, newPoints[i], {
                    width: _lastPoint3.width
                  });
                }

                newExpandingLine = setIn(_expandingLine4, ["points"], fixTwisted(newPoints));
              } else {
                return state;
              }

              return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex20]), newExpandingLine);
            }

          default:
            return state;
        }
      }

    case "OPEN_REGION_EDITOR":
      {
        // const { region } = action
        var _regionIndex21 = getRegionIndex(action.region);

        if (_regionIndex21 === null) return state;
        var newRegions = setIn(activeImage.regions.map(function (r) {
          return _objectSpread({}, r, {
            highlighted: false,
            editingLabels: false
          });
        }), [_regionIndex21], _objectSpread({}, (activeImage.regions || [])[_regionIndex21], {
          highlighted: true,
          editingLabels: true
        }));
        return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions"]), newRegions);
      }

    case "CLOSE_REGION_EDITOR":
      {
        // const { region } = action
        var _regionIndex22 = getRegionIndex(action.region);

        if (_regionIndex22 === null) return state;
        return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex22]), _objectSpread({}, (activeImage.regions || [])[_regionIndex22], {
          editingLabels: false
        }));
      }

    case "DELETE_REGION":
      {
        var _regionIndex23 = getRegionIndex(action.region);

        if (_regionIndex23 === null) return state;
        return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions"]), (activeImage.regions || []).filter(function (r) {
          return r.id !== action.region.id;
        }));
      }

    case "DELETE_REGION_POINT":
      {
        var _region9 = action.region,
            _subClassMenuData = action.subClassMenuData;

        var _regionIndex24 = getRegionIndex(_region9); // eslint-disable-next-line


        var _newPoints = _region9.points.filter(function (point, i) {
          if (i !== _subClassMenuData.pointIndex) return point;
        });

        return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions", _regionIndex24, "points"]), _newPoints);
      }

    case "DELETE_SELECTED_REGION":
      {
        return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions"]), (activeImage.regions || []).filter(function (r) {
          return !r.highlighted;
        }));
      }

    case "HEADER_BUTTON_CLICKED":
      {
        var buttonName = action.buttonName.toLowerCase();

        switch (buttonName) {
          case "prev":
            {
              if (currentImageIndex === null) return state;
              if (currentImageIndex === 0) return state;
              return setNewImage(state.images[currentImageIndex - 1], currentImageIndex - 1);
            }

          case "next":
            {
              if (currentImageIndex === null) return state;
              if (currentImageIndex === state.images.length - 1) return state;
              return setNewImage(state.images[currentImageIndex + 1], currentImageIndex + 1);
            }

          case "clone":
            {
              if (currentImageIndex === null) return state;
              if (currentImageIndex === state.images.length - 1) return state;
              return setIn(setNewImage(state.images[currentImageIndex + 1], currentImageIndex + 1), ["images", currentImageIndex + 1, "regions"], activeImage.regions);
            }

          case "settings":
            {
              return setIn(state, ["settingsOpen"], !state.settingsOpen);
            }

          case "help":
            {
              return state;
            }

          case "fullscreen":
            {
              return setIn(state, ["fullScreen"], true);
            }

          case "exit fullscreen":
          case "window":
            {
              return setIn(state, ["fullScreen"], false);
            }

          case "hotkeys":
            {
              return state;
            }

          case "exit":
          case "done":
            {
              return state;
            }

          default:
            return state;
        }
      }

    case "SELECT_TOOL":
      {
        if (action.selectedTool === "show-tags") {
          setInLocalStorage("showTags", !state.showTags);
          return setIn(state, ["showTags"], !state.showTags);
        } else if (action.selectedTool === "show-mask") {
          return setIn(state, ["showMask"], !state.showMask);
        }

        if (action.selectedTool === "modify-allowed-area" && !state.allowedArea) {
          state = setIn(state, ["allowedArea"], {
            x: 0,
            y: 0,
            w: 1,
            h: 1
          });
        }

        state = setIn(state, ["mode"], null);
        return setIn(state, ["selectedTool"], action.selectedTool);
      }

    case "CANCEL":
      {
        var _state3 = state,
            mode = _state3.mode;

        if (mode) {
          switch (mode.mode) {
            case "DRAW_EXPANDING_LINE":
            case "SET_EXPANDING_LINE_WIDTH":
            case "DRAW_POLYGON":
              {
                var _regionId11 = mode.regionId;
                return modifyRegion(_regionId11, null);
              }

            case "DRAW_POLYLINE":
              {
                var _regionId12 = mode.regionId;
                return modifyRegion(_regionId12, null);
              }

            case "DRAW_POLYLINE_BS":
              {
                var _regionId13 = mode.regionId;
                return modifyRegion(_regionId13, null);
              }

            case "MOVE_POLYGON_POINT":
            case "RESIZE_BOX":
            case "MOVE_REGION":
              {
                return setIn(state, ["mode"], null);
              }

            default:
              return state;
          }
        } // Close any open boxes


        var _regions4 = activeImage.regions;

        if (_regions4 && _regions4.some(function (r) {
          return r.editingLabels;
        })) {
          return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions"]), _regions4.map(function (r) {
            return _objectSpread({}, r, {
              editingLabels: false
            });
          }));
        } else if (_regions4) {
          return setIn(state, [].concat(_toConsumableArray(pathToActiveImage), ["regions"]), _regions4.map(function (r) {
            return _objectSpread({}, r, {
              highlighted: false
            });
          }));
        }

        break;
      }

    default:
      break;
  }

  return state;
});